@page {
    @bottom-center{
        content: counter(page) " / " counter(pages);
        font:13px "Myriad Pro","Lucida Grande",Lucida,Verdana,sans-serif;
    }
}
/* Main CSS from the Markdown Viewer Extension */
body,h1{
    color:#000
}
body{
    background-color:#fff;
    font:13px "Myriad Pro","Lucida Grande",Lucida,Verdana,sans-serif
}
a:link,a:visited{
    color:#00f;
    text-decoration:none
}
a:visited{
    color:#00a
}
a:active,a:hover{
    color:#f60;
    text-decoration:underline
}
* html code,* html pre{
    font-size:101%
}
code,pre{
    font-size:11px;
    font-family:monaco,courier,consolas,monospace
}
pre{
    border:1px solid #c7cfd5;
    background:#f1f5f9;
    margin:20px 0;
    padding:8px;
    text-align:left
}
hr,table{
    width:100%
}
hr{
    color:#919699;
    size:1;
    noshade:"noshade"
}
h1{
    margin-top:1em;
    margin-bottom:25px;
    font-size:30px
}
h1,h2,h3,h4,h5,h6{
    font-family:"Myriad Pro","Lucida Grande",Lucida,Verdana,sans-serif;
    font-weight:700
}
h2,table td,table th{
    border-bottom:1px solid #919699
}
h2{
    margin-top:2.5em;
    color:#000;
    padding-bottom:2px;
    font-size:24px
}
h3,h4,h5,h6{
    margin-top:2em;
    margin-bottom:.5em;
    font-size:17px;
    color:#000
}
h4,h5,h6{
    font-size:15px
}
h5,h6{
    margin-top:20px;
    padding:0;
    font-size:13px
}
h6{
    font-size:11px
}
p{
    margin-top:0;
    margin-bottom:10px
}
ol,ol ol,ul{
    margin:0 0 0 30px;
    padding:0 0 12px 6px
}
ul{
    list-style:square outside
}
li{
    margin-top:7px
}
ol,ol ol{
    list-style-type:decimal;
    list-style-position:outside
}
ol ol{
    list-style-type:lower-alpha;
    margin:7px 0 0 30px;
    padding:0 0 0 10px
}
ul ul{
    margin-left:40px;
    padding:0 0 0 6px
}
li>p{
    display:inline
}
li>a+p,li>p+p{
    display:block
}
table{
    border-top:1px solid #919699;
    border-left:1px solid #919699;
    border-spacing:0
}
table td,table th{
    font-size:12px;
    border-right:1px solid #919699
}
table th{
    background:#e2e2e2;
    padding:4px 8px
}
table th p{
    font-weight:700
}
table td{
    padding:8px;
    vertical-align:top
}
table td p,table th p{
    margin-bottom:0
}
table td p+p,table td p+p+p{
    margin-top:5px
}
form{
    margin:0
}
button{
    margin:3px 0 10px
}
input,select{
    vertical-align:middle;
    padding:0;
    margin:0 0 5px
}
select{
    margin:0 0 3px
}
textarea{
    margin:0 0 10px;
    width:100%
}
